.text
.globl	_OPENSSL_UplinkTable
.def	__$lazy1;	.scl	3;	.type	32;	.endef
.align	16
__$lazy1:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$1
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*4(%eax)
.def	__$lazy2;	.scl	3;	.type	32;	.endef
.align	16
__$lazy2:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$2
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*8(%eax)
.def	__$lazy3;	.scl	3;	.type	32;	.endef
.align	16
__$lazy3:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$3
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*12(%eax)
.def	__$lazy4;	.scl	3;	.type	32;	.endef
.align	16
__$lazy4:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$4
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*16(%eax)
.def	__$lazy5;	.scl	3;	.type	32;	.endef
.align	16
__$lazy5:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$5
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*20(%eax)
.def	__$lazy6;	.scl	3;	.type	32;	.endef
.align	16
__$lazy6:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$6
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*24(%eax)
.def	__$lazy7;	.scl	3;	.type	32;	.endef
.align	16
__$lazy7:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$7
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*28(%eax)
.def	__$lazy8;	.scl	3;	.type	32;	.endef
.align	16
__$lazy8:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$8
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*32(%eax)
.def	__$lazy9;	.scl	3;	.type	32;	.endef
.align	16
__$lazy9:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$9
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*36(%eax)
.def	__$lazy10;	.scl	3;	.type	32;	.endef
.align	16
__$lazy10:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$10
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*40(%eax)
.def	__$lazy11;	.scl	3;	.type	32;	.endef
.align	16
__$lazy11:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$11
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*44(%eax)
.def	__$lazy12;	.scl	3;	.type	32;	.endef
.align	16
__$lazy12:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$12
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*48(%eax)
.def	__$lazy13;	.scl	3;	.type	32;	.endef
.align	16
__$lazy13:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$13
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*52(%eax)
.def	__$lazy14;	.scl	3;	.type	32;	.endef
.align	16
__$lazy14:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$14
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*56(%eax)
.def	__$lazy15;	.scl	3;	.type	32;	.endef
.align	16
__$lazy15:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$15
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*60(%eax)
.def	__$lazy16;	.scl	3;	.type	32;	.endef
.align	16
__$lazy16:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$16
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*64(%eax)
.def	__$lazy17;	.scl	3;	.type	32;	.endef
.align	16
__$lazy17:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$17
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*68(%eax)
.def	__$lazy18;	.scl	3;	.type	32;	.endef
.align	16
__$lazy18:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$18
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*72(%eax)
.def	__$lazy19;	.scl	3;	.type	32;	.endef
.align	16
__$lazy19:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$19
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*76(%eax)
.def	__$lazy20;	.scl	3;	.type	32;	.endef
.align	16
__$lazy20:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$20
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*80(%eax)
.def	__$lazy21;	.scl	3;	.type	32;	.endef
.align	16
__$lazy21:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$21
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*84(%eax)
.def	__$lazy22;	.scl	3;	.type	32;	.endef
.align	16
__$lazy22:
	#ifdef __CET__

.byte	243,15,30,251
	#endif

	leal	_OPENSSL_UplinkTable,%eax
	pushl	$22
	pushl	%eax
	call	_OPENSSL_Uplink
	popl	%eax
	addl	$4,%esp
	jmp	*88(%eax)
.data
.align	4
_OPENSSL_UplinkTable:
.long	22
.long	__$lazy1
.long	__$lazy2
.long	__$lazy3
.long	__$lazy4
.long	__$lazy5
.long	__$lazy6
.long	__$lazy7
.long	__$lazy8
.long	__$lazy9
.long	__$lazy10
.long	__$lazy11
.long	__$lazy12
.long	__$lazy13
.long	__$lazy14
.long	__$lazy15
.long	__$lazy16
.long	__$lazy17
.long	__$lazy18
.long	__$lazy19
.long	__$lazy20
.long	__$lazy21
.long	__$lazy22
